@@ -1,5 +1,5 @@
 # -*- mode: python; coding: utf-8 -*-
-# :Progetto: vcpx -- bazaar-ng support using the bzrlib instead of the frontend
+# :Progetto: vcpx -- Bazaar support using the bzrlib instead of the frontend
 # :Creato:   Fri Aug 19 01:06:08 CEST 2005
 # :Autore:   Johan Rydberg <jrydberg@gnu.org>
 #            Jelmer Vernooij <jelmer@samba.org>
@@ -9,20 +9,18 @@
 #
 
 """
-This module implements the backends for Bazaar-NG.
+This module implements the backends for Bazaar.
 """
 
 __docformat__ = 'reStructuredText'
 
 
 from sys import version_info
-assert version_info >= (2,4), "Bazaar-NG backend requires Python 2.4"
+assert version_info >= (2,4), "Bazaar backend requires Python 2.4"
 del version_info
 
 from bzrlib import errors
-from bzrlib.add import smart_add_tree
 from bzrlib.bzrdir import BzrDir
-from bzrlib.delta import compare_trees
 from bzrlib.osutils import normpath, pathjoin
 from bzrlib.plugin import load_plugins
 
@@ -47,7 +45,7 @@ class BzrRepository(Repository):
     def create(self):
         """
         Create a branch with a working tree at the base directory. If the base
-        directory is inside a Bazaar-NG style "shared repository", it will use
+        directory is inside a Bazaar style "shared repository", it will use
         that to create a branch and working tree (make sure it allows working
         trees).
         """
@@ -195,7 +193,12 @@ class BzrWorkingDir(UpdatableSourceWorkingDir, SynchronizableTargetWorkingDir):
         self.log.debug("%s updated to %s",
                        ', '.join([e.name for e in changeset.entries]),
                        changeset.revision)
-        if (count != 1) or conflicts:
+        try:
+            pulled_revnos = count.new_revno - count.old_revno
+        except AttributeError:
+            # Prior to 0.15 pull returned a simple integer instead of a result object
+            pulled_revnos = count
+        if (pulled_revnos != 1) or conflicts:
             raise ChangesetApplicationFailure('unknown reason')
         return [] # No conflict handling yet
 
@@ -227,11 +230,11 @@ class BzrWorkingDir(UpdatableSourceWorkingDir, SynchronizableTargetWorkingDir):
     def _addPathnames(self, names):
         if len(names):
             names = [ pathjoin(self.repository.basedir, n) for n in names ]
-            smart_add_tree(self._working_tree, names, recurse=False)
+            self._working_tree.smart_add(names, recurse=False)
 
     def _addSubtree(self, subdir):
         subdir = pathjoin(self.repository.basedir, subdir)
-        added, ignored = smart_add_tree(self._working_tree, [subdir], recurse=True)
+        added, ignored = self._working_tree.smart_add([subdir], recurse=True)
 
         from vcpx.dualwd import IGNORED_METADIRS
 
@@ -252,7 +255,6 @@ class BzrWorkingDir(UpdatableSourceWorkingDir, SynchronizableTargetWorkingDir):
         from calendar import timegm  # like mktime(), but returns UTC timestamp
         from binascii import hexlify
         from re import search
-        from bzrlib.osutils import compact_date, rand_bytes
 
         logmessage = []
         if patchname:
@@ -269,23 +271,12 @@ class BzrWorkingDir(UpdatableSourceWorkingDir, SynchronizableTargetWorkingDir):
         timestamp = timegm(date.utctimetuple())
         timezone  = date.utcoffset().seconds + date.utcoffset().days * 24 * 3600
 
-        # Guess sane email address
-        email = search("<(.*@.*)>", author)
-        if email:
-            email = email.group(1)
-        else:
-            email = author
-        # Remove whitespace
-        email = ''.join(email.split())
-
         # Normalize file names
         if entries:
             entries = [normpath(entry) for entry in entries]
 
-        revision_id = "%s-%s-%s" % (email, compact_date(timestamp),
-                                    hexlify(rand_bytes(8)))
         self._working_tree.commit(logmessage, committer=author,
-                                  specific_files=entries, rev_id=revision_id,
+                                  specific_files=entries,
                                   verbose=self.repository.projectref().verbose,
                                   timestamp=timestamp, timezone=timezone)
 
