@@ -19,7 +19,11 @@ from vcpx.tzinfo import UTC
 
 
 class CvsRepository(CvspsRepository):
-    pass
+    def _load(self, project):
+        CvspsRepository._load(self, project)
+
+        tmc = project.config.get(self.name, 'trim-module-components', '0')
+        self.trim_module_components = int(tmc)
 
 
 def normalize_cvs_rev(rev):
@@ -89,12 +93,15 @@ def rev2branch(rev):
     return rev[0:-1]
 
 
-def changesets_from_cvslog(log, module, branch=None, entries=None, since=None, threshold=None):
+def changesets_from_cvslog(log, module, branch=None,
+                           entries=None, since=None, threshold=None,
+                           trim_module_components=0):
     """
     Parse CVS log.
     """
 
-    collected = ChangeSetCollector(log, module, branch, entries, since)
+    collected = ChangeSetCollector(log, module, branch, entries, since,
+                                   trim_module_components)
 
     last = None
 
@@ -173,7 +180,8 @@ class ChangeSetCollector(object):
     intra_sep = '-' * 28 + '\n'
     inter_sep = '=' * 77 + '\n'
 
-    def __init__(self, log, module, branch, entries, since):
+    def __init__(self, log, module, branch, entries, since,
+                 trim_module_components=0):
         """
         Initialize a ChangeSetCollector instance.
 
@@ -196,12 +204,21 @@ class ChangeSetCollector(object):
 
         self.log = getLogger('tailor.vcpx.cvs.collector')
 
+        self.trim_module_components = trim_module_components
+
         self.__parseCvsLog(branch, entries, since)
 
     def __iter__(self):
-        keys = self.changesets.keys()
+        # Since there can be duplicate keys, try to produce the right
+        # ordering taking into account the first action (thus ADDs
+        # will preceed UPDs...)
+        keys = []
+        for k,c in self.changesets.items():
+            action1 = len(c.entries)>0 and c.entries[0].action_kind or ' '
+            keys.append( (k[0], k[1], action1, k[2]) )
         keys.sort()
-        return iter([self.changesets[k] for k in keys])
+
+        return iter([self.changesets[(k[0], k[1], k[3])] for k in keys])
 
     def __collect(self, timestamp, author, changelog, entry, revision):
         """Register a change set about an entry."""
@@ -238,12 +255,23 @@ class ChangeSetCollector(object):
                 l = self.__lookahead.pop(0)
             else:
                 l = self.cvslog.readline()
-        while l.startswith('cvs rlog: Logging '):
+        # Some version of CVS emits the following with a different char-case
+        while l.lower().startswith('cvs rlog: logging '):
             currentdir = l[18:-1]
-            # If the directory starts with the module name, keep
-            # just the remaining part
             if currentdir.startswith(self.module):
+                # If the directory starts with the module name, keep
+                # just the remaining part
                 self.__currentdir = currentdir[len(self.module)+1:]
+            elif self.trim_module_components:
+                # This is a quick&dirty workaround to the CVS modules
+                # issue: if, by some heuristic, the user tells how
+                # many components to cut off...
+                parts = currentdir.split('/')
+                if len(parts)>self.trim_module_components:
+                    parts = parts[self.trim_module_components:]
+                else:
+                    parts = []
+                self.__currentdir = '/'.join(parts)
             else:
                 # strip away first component, the name of the product
                 slash = currentdir.find('/')
@@ -335,12 +363,12 @@ class ChangeSetCollector(object):
         """Parse a complete CVS log."""
 
         from os.path import split, join
-        import sre
+        from re import compile
         from time import strptime
         from datetime import datetime
         from vcpx.changes import Changeset
 
-        revcount_regex = sre.compile('\\bselected revisions:\\s*(\\d+)\\b')
+        revcount_regex = compile('\\bselected revisions:\\s*(\\d+)\\b')
 
         self.__currentdir = None
 
@@ -640,7 +668,8 @@ class CvsWorkingDir(CvspsWorkingDir):
                                       branch,
                                       CvsEntries(self.repository.rootdir),
                                       since,
-                                      self.repository.changeset_threshold)
+                                      self.repository.changeset_threshold,
+                                      self.repository.trim_module_components)
 
     def _checkoutUpstreamRevision(self, revision):
         """
