@@ -386,6 +386,18 @@ class CvspsWorkingDir(UpdatableSourceWorkingDir,
             deldir.action_kind = deldir.DELETED
             self.log.info("registering %s directory deletion", path)
 
+        # Make sure all files are present on disk: CVS update does not
+        # create them nor reports an error if the files have been
+        # completely removed from the cvs repository. So loop over the
+        # entries and verify the presence of added/changed ones.
+
+        for entry in changeset.entries:
+            if (entry.action_kind in (entry.ADDED, entry.UPDATED)
+                and not exists(join(self.repository.basedir, entry.name))):
+                self.log.warning("Ignoring entry %s, CVS source no "
+                                 "longer knows about it.", entry.name)
+                changeset.entries.remove(entry)
+
     def _checkoutUpstreamRevision(self, revision):
         """
         Concretely do the checkout of the upstream sources. Use
@@ -535,7 +547,11 @@ class CvspsWorkingDir(UpdatableSourceWorkingDir,
         if not found:
             raise TargetInitializationFailure(
                 "Something went wrong: unable to determine the exact upstream "
-                "revision of the checked out tree in '%s'" % self.repository.basedir)
+                "revision of the checked out tree in '%s'. Either you specified "
+                "the wrong initial timestamp, or you are checking out a "
+                "composition of 'CVS modules' and Tailor does not support them; "
+                "see the option 'trim-module-components' for a possible "
+                "workaround." % self.repository.basedir)
         else:
             self.log.info("Working copy up to revision %s", last.revision)
 
@@ -761,7 +777,7 @@ class CvspsWorkingDir(UpdatableSourceWorkingDir,
         self._removePathnames([oldname])
         self._addPathnames([newname])
 
-    def _tag(self, tagname):
+    def _tag(self, tagname, date, author):
         """
         Apply a tag.
         """
