@@ -26,7 +26,7 @@ class GitRepository(Repository):
     def _load(self, project):
         Repository._load(self, project)
         self.EXECUTABLE = project.config.get(self.name, 'git-command', 'git')
-	self.overwrite_tags = project.config.get(self.name, 'overwrite-tags', False)
+        self.overwrite_tags = project.config.get(self.name, 'overwrite-tags', False)
         self.parent_repo = project.config.get(self.name, 'parent-repo')
         self.branch_point = project.config.get(self.name, 'branchpoint', 'HEAD')
         self.branch_name = project.config.get(self.name, 'branch')
@@ -43,12 +43,24 @@ class GitRepository(Repository):
 
         self.env = {}
 
+        # XXX: this seems plain wrong to me [who does not know git at
+        # all!]: why storagedir gets set to repository here? and why
+        # the need for the GIT_DIR envvar? I fail to see when
+        # storagedir should be different from '.git', the default,
+        # given how it's being used by this class! The same with
+        # GIT_INDEX_FILE, that accordingly with the man page defaults
+        # to ".git/index" anyway...
+        #
+        self.storagedir = self.METADIR
         if self.repository:
-            self.storagedir = self.repository
-            self.env['GIT_DIR'] = self.storagedir
+            from os.path import join
+
+            #self.storagedir = self.repository
+            #self.env['GIT_DIR'] = self.storagedir
+            self.env['GIT_DIR'] = join(self.basedir, self.METADIR)
             self.env['GIT_INDEX_FILE'] = self.METADIR + '/index'
-        else:
-            self.storagedir = self.METADIR
+        #else:
+        #    self.storagedir = self.METADIR
 
     def runCommand(self, cmd, exception=Exception, pipe=True):
         """
