@@ -46,8 +46,13 @@ class GitTargetWorkingDir(SynchronizableTargetWorkingDir):
         Records a sequence of filesystem objects as updated.
         """
 
-        # can we assume we don't have directories in the list ?
-        self.repository.runCommand(['update-index'] + names)
+        from os.path import join, isdir
+
+        # can we assume we don't have directories in the list ?  Nope.
+
+        notdirs = [n for n in names if not isdir(join(self.repository.basedir, n))]
+        if notdirs:
+            self.repository.runCommand(['update-index'] + notdirs)
 
     def __parse_author(self, author):
         """
@@ -66,7 +71,7 @@ class GitTargetWorkingDir(SynchronizableTargetWorkingDir):
         if not name:
             name = AUTHOR
         if not email:
-            email = "%s@%s" % (AUTHOR, HOST)
+            email = "%s@%s" % (name, HOST)
         return (name, email)
 
     def _commit(self, date, author, patchname, changelog=None, entries=None,
@@ -77,6 +82,12 @@ class GitTargetWorkingDir(SynchronizableTargetWorkingDir):
 
         from os import environ
 
+        try:
+            self.repository.runCommand(['status'])
+        except Exception, e:
+            self.log.info("git-status returned an error---assuming nothing to do")
+            return
+
         encode = self.repository.encode
 
         logmessage = []
@@ -147,7 +158,7 @@ class GitTargetWorkingDir(SynchronizableTargetWorkingDir):
             else:
                 self.repository.runCommand(['update-ref', refname, commitid])
 
-    def _tag(self, tag):
+    def _tag(self, tag, date, author):
 
         # in single-repository mode, only update the relevant branch
         if self.repository.branch_name:
