@@ -38,6 +38,39 @@ class SvnRepository(Repository):
         self.use_limit = cget(self.name, 'use-limit', True)
         self.trust_root = cget(self.name, 'trust-root', False)
         self.ignore_externals = cget(self.name, 'ignore-externals', True)
+        self.commit_all_files = cget(self.name, 'commit-all-files', True)
+        self.tags_path = cget(self.name, 'svn-tags', '/tags')
+        self.branches_path = cget(self.name, 'svn-branches', '/branches')
+        self._setupTagsDirectory = None
+
+    def setupTagsDirectory(self):
+        if self._setupTagsDirectory == None:
+            self._setupTagsDirectory = False
+            if self.module and self.module <> '/':
+
+                # Check the existing tags directory
+                cmd = self.command("ls")
+                svnls = ExternalCommand(command=cmd)
+                svnls.execute(self.repository + self.tags_path)
+                if svnls.exit_status:
+                    # create it, if not exist
+                    cmd = self.command("mkdir", "-m",
+                                       "This directory will host the tags")
+                    svnmkdir = ExternalCommand(command=cmd)
+                    svnmkdir.execute(self.repository + self.tags_path)
+                    if svnmkdir.exit_status:
+                        raise TargetInitializationFailure(
+                                    "Was not able to create tags directory '%s'"
+                                    % self.tags_path)
+                else:
+                    self.log.debug("Directory '%s' already exists"
+                                   % self.tags_path)
+                self._setupTagsDirectory = True
+            else:
+                self.log.debug("Tags needs module setup other than '/'")
+
+        return self._setupTagsDirectory
+
 
     def _validateConfiguration(self):
         from vcpx.config import ConfigurationError
@@ -69,6 +102,16 @@ class SvnRepository(Repository):
                            self.module, self.name)
             self.module = '/' + self.module
 
+        if not self.tags_path.startswith('/'):
+            self.log.debug("Prepending '/' to svn-tags %r in %r",
+                           self.tags_path, self.name)
+            self.tags_path = '/' + self.tags_path
+
+        if not self.branches_path.startswith('/'):
+            self.log.debug("Prepending '/' to svn-branches %r in %r",
+                           self.branches_path, self.name)
+            self.branches_path = '/' + self.branches_path
+
     def create(self):
         """
         Create a local SVN repository, if it does not exist, and configure it.
@@ -101,28 +144,47 @@ class SvnRepository(Repository):
                                                   "svn repository at %r" %
                                                   self.repository)
         if self.use_propset:
-            hookname = join(repodir, 'hooks', 'pre-revprop-change')
-            if platform == 'win32':
-                hookname += '.bat'
-            if not exists(hookname):
-                prehook = open(hookname, 'w')
-                if platform <> 'win32':
-                    prehook.write('#!/bin/sh\n')
-                prehook.write('exit 0\n')
-                prehook.close()
-                if platform <> 'win32':
-                    from os import chmod
-                    chmod(hookname, 0755)
+            if not self.repository.startswith('file:///'):
+                self.log.warning("Repository is remote, cannot verify if it "
+                                 "has the 'pre-revprop-change' hook active, needed "
+                                 "by 'use-propset=True'. Assuming it does...")
+            else:
+                repodir = self.repository[7:]
+                hookname = join(repodir, 'hooks', 'pre-revprop-change')
+                if platform == 'win32':
+                    hookname += '.bat'
+                if not exists(hookname):
+                    prehook = open(hookname, 'w')
+                    if platform <> 'win32':
+                        prehook.write('#!/bin/sh\n')
+                    prehook.write('exit 0\n')
+                    prehook.close()
+                    if platform <> 'win32':
+                        from os import chmod
+                        chmod(hookname, 0755)
 
         if self.module and self.module <> '/':
             cmd = self.command("ls")
             svnls = ExternalCommand(command=cmd)
             svnls.execute(self.repository + self.module)
             if svnls.exit_status:
+
+                paths = []
+
+                # Auto detect missing "branches/"
+                if self.module.startswith(self.branches_path + '/'):
+                    path = self.repository + self.branches_path
+                    cmd = self.command("ls")
+                    svnls = ExternalCommand(command=cmd)
+                    svnls.execute(path)
+                    if svnls.exit_status:
+                        paths.append(path)
+
+                paths.append(self.repository + self.module)
                 cmd = self.command("mkdir", "-m",
                                    "This directory will host the upstream sources")
                 svnmkdir = ExternalCommand(command=cmd)
-                svnmkdir.execute(self.repository + self.module)
+                svnmkdir.execute(paths)
                 if svnmkdir.exit_status:
                     raise TargetInitializationFailure("Was not able to create the "
                                                       "module %r, maybe more than "
@@ -236,10 +298,23 @@ def changesets_from_svnlog(log, repository, chunksize=2**15):
                             return True
                     return False
 
+                # Find renames from deleted directories:
+                # $ svn mv dir/a.txt a.txt
+                # $ svn del dir
+                def check_renames_from_dir(name):
+                    for e in mv_or_cp.values():
+                        if e.old_name.startswith(name+'/'):
+                            e.action_kind = e.RENAMED
+
                 entries = []
+                entries2 = []
                 for e in self.current['entries']:
-                    if e.action_kind==e.DELETED and mv_or_cp.has_key(e.name):
-                        mv_or_cp[e.name].action_kind = e.RENAMED
+                    if e.action_kind==e.DELETED:
+                        if mv_or_cp.has_key(e.name):
+                            mv_or_cp[e.name].action_kind = e.RENAMED
+                        else:
+                            check_renames_from_dir(e.name)
+                            entries2.append(e)
                     elif e.action_kind=='R':
                         # In svn parlance, 'R' means Replaced: a typical
                         # scenario is
@@ -248,8 +323,10 @@ def changesets_from_svnlog(log, repository, chunksize=2**15):
                         #   $ svn add a.txt
                         if mv_or_cp.has_key(e.name):
                             mv_or_cp[e.name].action_kind = e.RENAMED
+                        else:
+                            check_renames_from_dir(e.name)
                         e.action_kind = e.ADDED
-                        entries.append(e)
+                        entries2.append(e)
                     elif parent_was_copied(e.name):
                         if e.action_kind != e.DELETED:
                             e.action_kind = e.ADDED
@@ -257,6 +334,10 @@ def changesets_from_svnlog(log, repository, chunksize=2**15):
                     else:
                         entries.append(e)
 
+                # Changes sort: first MODIFY|ADD|RENAME, than REPLACE|DELETE
+                for e in entries2:
+                    entries.append(e)
+
                 svndate = self.current['date']
                 # 2004-04-16T17:12:48.000000Z
                 y,m,d = map(int, svndate[:10].split('-'))
@@ -323,7 +404,7 @@ class SvnWorkingDir(UpdatableSourceWorkingDir, SynchronizableTargetWorkingDir):
         else:
             sincerev = 0
 
-        cmd = self.repository.command("log", "--verbose", "--xml",
+        cmd = self.repository.command("log", "--verbose", "--xml", "--non-interactive",
                                       "--revision", "%d:HEAD" % (sincerev+1))
         svnlog = ExternalCommand(cwd=self.repository.basedir, command=cmd)
         log = svnlog.execute('.', stdout=PIPE, TZ='UTC0')[0]
@@ -407,7 +488,9 @@ class SvnWorkingDir(UpdatableSourceWorkingDir, SynchronizableTargetWorkingDir):
 
         lastok = self.repository.repository
         if not self.repository.trust_root:
-            cmd = self.repository.command("ls")
+            # Use --non-interactive, so that it fails if credentials
+            # are needed.
+            cmd = self.repository.command("ls", "--non-interactive")
             svnls = ExternalCommand(command=cmd)
 
             # First verify that we have a valid repository
@@ -447,7 +530,7 @@ class SvnWorkingDir(UpdatableSourceWorkingDir, SynchronizableTargetWorkingDir):
         if revision == 'INITIAL':
             initial = True
             cmd = self.repository.command("log", "--verbose", "--xml",
-                                          "--stop-on-copy",
+                                          "--non-interactive", "--stop-on-copy",
                                           "--revision", "1:HEAD")
             if self.repository.use_limit:
                 cmd.extend(["--limit", "1"])
@@ -467,7 +550,7 @@ class SvnWorkingDir(UpdatableSourceWorkingDir, SynchronizableTargetWorkingDir):
         else:
             initial = False
 
-        if not exists(join(self.repository.basedir, '.svn')):
+        if not exists(join(self.repository.basedir, self.repository.METADIR)):
             self.log.debug("Checking out a working copy")
 
             cmd = self.repository.command("co", "--quiet")
@@ -493,6 +576,7 @@ class SvnWorkingDir(UpdatableSourceWorkingDir, SynchronizableTargetWorkingDir):
             if revision=='HEAD':
                 revision = 'COMMITTED'
             cmd = self.repository.command("log", "--verbose", "--xml",
+                                          "--non-interactive",
                                           "--revision", revision)
             svnlog = ExternalCommand(cwd=self.repository.basedir, command=cmd)
             out, err = svnlog.execute(stdout=PIPE, stderr=PIPE)
@@ -520,14 +604,71 @@ class SvnWorkingDir(UpdatableSourceWorkingDir, SynchronizableTargetWorkingDir):
                                       "--non-recursive")
         ExternalCommand(cwd=self.repository.basedir, command=cmd).execute(names)
 
+    def _propsetRevision(self, out, command, date, author):
+
+        from re import search
+
+        encode = self.repository.encode
+
+        line = out.readline()
+        if not line:
+            # svn did not find anything to commit
+            self.log.warning('svn did not find anything to commit')
+            return
+
+        # Assume svn output the revision number in the last output line
+        while line:
+            lastline = line
+            line = out.readline()
+        revno = search('\d+', lastline)
+        if revno is None:
+            out.seek(0)
+            raise ChangesetApplicationFailure("%s wrote unrecognizable "
+                                              "revision number:\n%s" %
+                                              (str(command), out.read()))
+
+        revision = revno.group(0)
+
+        if self.repository.use_propset:
+
+            cmd = self.repository.command("propset", "%(propname)s",
+                                          "--quiet", "--revprop",
+                                          "--revision", revision)
+            pset = ExternalCommand(cwd=self.repository.basedir, command=cmd)
+            if self.repository.propset_date:
+                date = date.astimezone(UTC).replace(microsecond=0, tzinfo=None)
+                pset.execute(date.isoformat()+".000000Z", propname='svn:date')
+            pset.execute(encode(author), propname='svn:author')
+
+        return revision
+
+    def _tag(self, tag, date, author):
+        """
+        TAG current revision.
+        """
+        if self.repository.setupTagsDirectory():
+            src = self.repository.repository + self.repository.module
+            dest = self.repository.repository + self.repository.tags_path \
+                                              + '/' + tag.replace('/', '_')
+
+            cmd = self.repository.command("copy", src, dest, "-m", tag)
+            svntag = ExternalCommand(cwd=self.repository.basedir, command=cmd)
+            out, err = svntag.execute(stdout=PIPE, stderr=PIPE)
+
+            if svntag.exit_status:
+                raise ChangesetApplicationFailure("%s returned status %d saying\n%s"
+                                                  % (str(svntag),
+                                                     svntag.exit_status,
+                                                     err.read()))
+
+            self._propsetRevision(out, svntag, date, author)
+
     def _commit(self, date, author, patchname, changelog=None, entries=None,
                 tags = [], isinitialcommit = False):
         """
         Commit the changeset.
         """
 
-        from re import search
-
         encode = self.repository.encode
 
         logmessage = []
@@ -555,7 +696,7 @@ class SvnWorkingDir(UpdatableSourceWorkingDir, SynchronizableTargetWorkingDir):
         cmd = self.repository.command("commit", "--file", rontf.name)
         commit = ExternalCommand(cwd=self.repository.basedir, command=cmd)
 
-        if not entries:
+        if not entries or self.repository.commit_all_files:
             entries = ['.']
 
         out, err = commit.execute(entries, stdout=PIPE, stderr=PIPE)
@@ -565,33 +706,12 @@ class SvnWorkingDir(UpdatableSourceWorkingDir, SynchronizableTargetWorkingDir):
                                               % (str(commit),
                                                  commit.exit_status,
                                                  err.read()))
-        line = out.readline()
-        if not line:
+
+        revision = self._propsetRevision(out, commit, date, author)
+        if not revision:
             # svn did not find anything to commit
             return
 
-        # Assume svn output the revision number in the last output line
-        while line:
-            lastline = line
-            line = out.readline()
-        revno = search('\d+', lastline)
-        if revno is None:
-            out.seek(0)
-            raise ChangesetApplicationFailure("%s wrote unrecognizable "
-                                              "revision number:\n%s" %
-                                              (str(commit), out.read()))
-        revision = revno.group(0)
-
-        if self.repository.use_propset:
-            cmd = self.repository.command("propset", "%(propname)s",
-                                          "--quiet", "--revprop",
-                                          "--revision", revision)
-            pset = ExternalCommand(cwd=self.repository.basedir, command=cmd)
-            if self.repository.propset_date:
-                date = date.astimezone(UTC).replace(microsecond=0, tzinfo=None)
-                pset.execute(date.isoformat()+".000000Z", propname='svn:date')
-            pset.execute(encode(author), propname='svn:author')
-
         cmd = self.repository.command("update", "--quiet")
         if self.repository.ignore_externals:
             cmd.append("--ignore-externals")
@@ -614,7 +734,9 @@ class SvnWorkingDir(UpdatableSourceWorkingDir, SynchronizableTargetWorkingDir):
         """
 
         from os import rename
-        from os.path import join, exists
+        from os.path import join, exists, isdir
+        from time import sleep
+        from datetime import datetime
 
         # --force in case the file has been changed and moved in one revision
         cmd = self.repository.command("mv", "--quiet", "--force")
@@ -636,6 +758,18 @@ class SvnWorkingDir(UpdatableSourceWorkingDir, SynchronizableTargetWorkingDir):
                                   newpath, oldpath)
                 raise
             unmoved = True
+
+        # Ticket #135: Need a timediff between rsync and directory move
+        if isdir(oldpath):
+            now = datetime.now()
+            if hasattr(self, '_last_svn_move'):
+                last = self._last_svn_move
+            else:
+                last = now
+            if not (now-last).seconds:
+                sleep(1)
+            self._last_svn_move = now
+
         move = ExternalCommand(cwd=self.repository.basedir, command=cmd)
         out, err = move.execute(oldname, newname, stdout=PIPE, stderr=PIPE)
         if move.exit_status:
@@ -662,7 +796,7 @@ class SvnWorkingDir(UpdatableSourceWorkingDir, SynchronizableTargetWorkingDir):
 
         from os.path import join, exists
 
-        if not self.repository.repository or exists(join(self.repository.basedir, '.svn')):
+        if not self.repository.repository or exists(join(self.repository.basedir, self.repository.METADIR)):
             return
 
         cmd = self.repository.command("co", "--quiet")
@@ -680,7 +814,7 @@ class SvnWorkingDir(UpdatableSourceWorkingDir, SynchronizableTargetWorkingDir):
 
         from os.path import exists, join
 
-        if not exists(join(self.repository.basedir, '.svn')):
+        if not exists(join(self.repository.basedir, self.repository.METADIR)):
             raise TargetInitializationFailure("'%s' needs to be an SVN working copy already under SVN" % self.repository.basedir)
 
         SynchronizableTargetWorkingDir._initializeWorkingDir(self)
