@@ -11,7 +11,7 @@ Implement the frontend functionalities.
 
 __docformat__ = 'reStructuredText'
 
-__version__ = '0.9.28'
+__version__ = '0.9.30'
 
 from logging import getLogger
 from optparse import OptionParser, OptionGroup, Option
@@ -135,7 +135,7 @@ class Tailorizer(Project):
 
         pname_format = pconfig('patch-name-format', raw=True)
         if pname_format is not None:
-            SynchronizableTargetWorkingDir.PATCH_NAME_FORMAT = pname_format.strip() or '%(revision)s'
+            SynchronizableTargetWorkingDir.PATCH_NAME_FORMAT = pname_format.strip()
         SynchronizableTargetWorkingDir.REMOVE_FIRST_LOG_LINE = pconfig('remove-first-log-line')
         Changeset.REFILL_MESSAGE = pconfig('refill-changelogs')
 
