@@ -126,7 +126,7 @@ class SynchronizableTargetWorkingDir(WorkingDir):
                          entries, tags = changeset.tags)
             if changeset.tags:
                 for tag in changeset.tags:
-                    self._tag(tag)
+                    self._tag(tag, changeset.date, changeset.author)
         finally:
             signal(SIGINT, previous)
 
@@ -429,8 +429,9 @@ class SynchronizableTargetWorkingDir(WorkingDir):
         of each entry.
         """
 
-        from os import rename
-        from os.path import split, join, exists
+        from os import rename, walk
+        from shutil import rmtree
+        from os.path import split, join, exists, isdir
 
         added = []
         for e in entries:
@@ -445,6 +446,7 @@ class SynchronizableTargetWorkingDir(WorkingDir):
                 parents.reverse()
                 self._addPathnames(parents)
 
+            other = False
             if self.shared_basedirs:
                 # Check to see if the oldentry is still there. If it is,
                 # that probably means one thing: it's been moved and then
@@ -464,19 +466,73 @@ class SynchronizableTargetWorkingDir(WorkingDir):
                 # rename the new one, perform the target system rename
                 # and replace back the real content (it may be a
                 # renamed+edited event).
-                renamed = False
+
+                # Hide the real new file from rename
                 absnew = join(self.repository.basedir, e.name)
                 renamed = exists(absnew)
                 if renamed:
                     rename(absnew, absnew + '-TAILOR-HACKED-TEMP-NAME')
 
+                # If 'absold' exist, then the file was moved and replaced
+                # with an other file. Hide the other file from rename.
+                absold = join(self.repository.basedir, e.old_name)
+                other = exists(absold)
+                if other:
+                    rename(absold, absold + '-TAILOR-HACKED-OTHER-NAME')
+
+                # Restore the old file from backup.
+                oldfile = exists(absold + '-TAILOR-HACKED-OLD-NAME')
+                if oldfile:
+                    rename(absold + '-TAILOR-HACKED-OLD-NAME', absold)
+
             try:
                 self._renamePathname(e.old_name, e.name)
             finally:
+
+                # Restore other NEW target
+                if other:
+                    rename(absold + '-TAILOR-HACKED-OTHER-NAME', absold)
+
                 if renamed:
                     if self.shared_basedirs:
                         rename(absold + '-TAILOR-HACKED-TEMP-NAME', absold)
                     else:
+
+                        # before rsync      after rsync      the HACK            after "svn mv"           result
+                        # /basedir          /basedir         /basedir            /basedir                 /basedir
+                        # |                 |                |                   |                        |
+                        # +- /dirold        +- /dirold       +- /dirold          +- /dirnew        move   |
+                        #    |              |  |             |  |                |  |~~~~~~        hack   |
+                        #    +- /.svn       |  +- /.svn      |  +- /.svn         |  +- /.svn  >-------+   |
+                        #    |              |  |             |  |                |  |                 |   |
+                        #    +- /subdir     |  +- /subdir    |  +- /subdir       |  +- /subdir        v   |
+                        #       |           |     |          |     |             |     |              |   |
+                        #       +- /.svn    |     +- /.svn   |     +- /.svn      |     +- /.svn  >--+ |   |
+                        #                   |                |                   |                  | |   |
+                        #                   +- /dirnew       +- /dirnew-HACKED   +- /dirnew-HACKED  v |   +- /dirnew
+                        #                      |~~~~~~          |      ~~~~~~~      |               | |      |
+                        #                      |                |                   |               +-|--->  +- /.svn
+                        #                      |                |                   |                 |      |   ~~~~
+                        #                      +- /subdir       +- /subdir          +- /subdir        |      +- /subdir
+                        #                          ~~~~~~                                             |        |
+                        #                                                                             +----->  +- /.svn
+                        #                                                                                          ~~~~
+
+                        # Ticket #65, #125
+                        # If the target reposity has files in subdirectory,
+                        # then remove the complete dir.
+                        # But keep the dir '.svn', '_CVS', or what ever
+                        if isdir(absnew):
+                            if self.repository.METADIR <> None:
+                                for root, dirs, files in walk(absnew):
+                                    if self.repository.METADIR in dirs:
+                                        dirs.remove(self.repository.METADIR)  # don't visit SVN directories
+                                        svnnew = join(root, self.repository.METADIR)
+                                        hacked = join(absnew + '-TAILOR-HACKED-TEMP-NAME' + root[len(absnew):], self.repository.METADIR)
+                                        rename(svnnew, hacked)
+
+                            rmtree(absnew)
+
                         rename(absnew + '-TAILOR-HACKED-TEMP-NAME', absnew)
 
     def _renamePathname(self, oldname, newname):
@@ -550,7 +606,7 @@ class SynchronizableTargetWorkingDir(WorkingDir):
 
         if changeset.tags:
             for tag in changeset.tags:
-                self._tag(tag)
+                self._tag(tag, changeset.date, author)
 
         self._dismissChangeset(changeset)
 
@@ -567,7 +623,7 @@ class SynchronizableTargetWorkingDir(WorkingDir):
 
         self._addSubtree('.')
 
-    def _tag(self, tagname):
+    def _tag(self, tagname, date, author):
         """
         Tag the current version, if the VC type supports it, otherwise
         do nothing.
